import logging
import os
from logging.handlers import TimedRotatingFileHandler

log = logging.getLogger('server')

PATH = os.path.dirname(os.path.abspath(__file__))
PATH = os.path.join(PATH, 'app.main.log')

formatter = logging.Formatter("%(asctime)s - %(levelname)-8s - %(module)s - %(message)s")


trf_handler = TimedRotatingFileHandler(PATH, when='midnight', encoding='utf-8')
trf_handler.setLevel(logging.DEBUG)
trf_handler.setFormatter(formatter)

log.addHandler(trf_handler)
log.setLevel(logging.DEBUG)

if __name__ == '__main__':
    console = logging.StreamHandler()
    console.setLevel(logging.DEBUG)
    console.setFormatter(formatter)
    log.addHandler(console)
    log.info('Тестовый запуск')



"""Утилиты"""
import logging
import os
import sys
import json
import traceback
from datetime import datetime, date
from functools import wraps
from json import JSONEncoder

sys.path.append(os.path.join(os.getcwd(), '..'))
from .variables import MAX_PACKAGE_LENGTH, ENCODING
from errors import IncorrectDataRecivedError, NonDictInputError


def log(func):
    @wraps(func)
    def wrap(*args, **kwargs):
        script_name = sys.argv[0].split('/')[-1]

        if script_name == 'client.py':
            UTILS_LOGGER = logging.getLogger('client')
        elif script_name == 'server.py':
            UTILS_LOGGER = logging.getLogger('server')
        f = func(*args, **kwargs)
        UTILS_LOGGER.info(f'Вызванна функция {func.__name__} c аргументами {args}, {kwargs}')
        UTILS_LOGGER.info(
            f'Функция {func.__name__} вызвана из функции {traceback.format_stack()[0].strip().split()[-1]}')
        return f

    return wrap


@log
def get_msg(sock):
    encoded_response = sock.recv(MAX_PACKAGE_LENGTH)
    if isinstance(encoded_response, bytes):
        json_response = encoded_response.decode(ENCODING)
        response = json.loads(json_response)
        if isinstance(response, dict):
            return response
        else:
            raise IncorrectDataRecivedError
    else:
        raise IncorrectDataRecivedError


@log
def send_msg(sock, msg):
    if not isinstance(msg, dict):
        raise NonDictInputError
    js_message = json.dumps(msg)
    encoded_message = js_message.encode(ENCODING)
    sock.send(encoded_message)



import sys

from PyQt5.QtWidgets import QDialog, QLabel, QComboBox, QPushButton, QApplication
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QStandardItemModel, QStandardItem

sys.path.append('/')

from server_db import ServerStorage


class AddContactDialog(QDialog):
    """Окно добавления пользователя."""
    def __init__(self, database, username):
        super().__init__()
        self.username = username
        self.db = database

        self.setFixedSize(280, 100)
        self.setWindowTitle('Выбор контакта для добавления')

        self.setAttribute(Qt.WA_DeleteOnClose)
        self.setModal(True)

        self.selector_label = QLabel('Выберите контакт для добавления', self)
        self.selector_label.setFixedSize(260, 20)
        self.selector_label.move(40, 5)

        self.selector = QComboBox(self)
        self.selector.setFixedSize(260, 20)
        self.selector.move(10, 30)

        self.refresh_btn = QPushButton('Обновить', self)
        self.refresh_btn.setFixedSize(80, 30)
        self.refresh_btn.move(10, 60)

        self.add_btn = QPushButton('Добавить', self)
        self.add_btn.setFixedSize(80, 30)
        self.add_btn.move(100, 60)

        self.cancel_btn = QPushButton('Отмена', self)
        self.cancel_btn.setFixedSize(80, 30)
        self.cancel_btn.move(190, 60)
        self.cancel_btn.clicked.connect(self.close)

        self.client_contacts()
        self.refresh_btn.clicked.connect(self.client_contacts)


    def client_contacts(self):
        """Добавляет контакт в список контактов."""
        self.selector.clear()
        contact_list = set(self.db.get_all_contacts_login())
        contact_list = list(contact_list)
        contact_list.remove(self.username)
        self.selector.addItems(contact_list)


if __name__ == '__main__':
    db = ServerStorage()
    app = QApplication([])
    dial = AddContactDialog(db, 'test1')
    app.exec_()



"""Unit-тесты утилит"""

import sys
import os
import unittest
import json
from socket import socket, AF_INET, SOCK_STREAM

sys.path.append(os.path.join(os.getcwd(), '..'))
from common.variables import RESPONSE, ERROR, USER, ACCOUNT_NAME, TIME, ACTION, PRESENCE, ENCODING, MAX_PACKAGE_LENGTH, \
    DEFAULT_PORT, DEFAULT_IP_ADDRESS, MAX_CONNECTIONS
from common.utils import get_msg, send_msg


class TestClass(unittest.TestCase):
    msg_ok = {ACTION: PRESENCE,
              TIME: 1,
              USER: {
                  ACCOUNT_NAME: 'Guest'
              }
              }
    response_ok = {RESPONSE: 200}
    response_error = {RESPONSE: 400,
                      ERROR: 'Bad Request'}

    server_socket = None
    client_socket = None

    def setUp(self) -> None:
        self.server_socket = socket(AF_INET, SOCK_STREAM)
        self.server_socket.bind((DEFAULT_IP_ADDRESS, DEFAULT_PORT))
        self.server_socket.listen(MAX_CONNECTIONS)

        self.client_socket = socket(AF_INET, SOCK_STREAM)
        self.client_socket.connect((DEFAULT_IP_ADDRESS, DEFAULT_PORT))
        self.client, self.client_addres = self.server_socket.accept()

    def tearDown(self) -> None:
        self.client.close()
        self.client_socket.close()
        self.server_socket.close()

    def test_wrong_msg_from_client(self):
        self.assertRaises(TypeError, send_msg, self.client_socket, 'не словарь')

    def test_client_server_msg(self):
        send_msg(self.client_socket, self.msg_ok)
        response = self.client.recv(MAX_PACKAGE_LENGTH)
        response = json.loads(response.decode(ENCODING))
        self.assertEqual(self.msg_ok, response)

    def test_msg_from_server_not_dict(self):
        message = json.dumps('не словарь')
        self.client.send(message.encode(ENCODING))
        self.assertRaises(ValueError, get_msg, self.client_socket)

    def test_get_200_from_server(self):
        msg = json.dumps(self.response_ok)
        self.client.send(msg.encode(ENCODING))
        response = get_msg(self.client_socket)
        self.assertEqual(self.response_ok, response)

    def test_get_400_from_server(self):
        msg = json.dumps(self.response_error)
        self.client.send(msg.encode(ENCODING))
        response = get_msg(self.client_socket)
        self.assertEqual(self.response_error, response)


if __name__ == '__main__':
    unittest.main()



DEFAULT_PORT = 7777
DEFAULT_IP_ADDRESS = '127.0.0.1'
MAX_CONNECTIONS = 5
MAX_PACKAGE_LENGTH = 1024
ENCODING = 'utf-8'
DEFAULT_CLIENT_MODE = 'listen'
CLIENTS_LISTEN_NUM = 2
CLIENTS_SEND_NUM = 2

ACTION = 'action'
TIME = 'time'
USER = 'user'
ACCOUNT_NAME = 'account_name'
FROM = 'from'
TO = 'to'
CONTACT_NAME = 'contact_name'
PUBLIC_KEY = 'public_key'
PASSWORD_HASH = 'password_hash'

STATE = 'state'
AUTH = 'auth'
REG = 'reg'

PRESENCE = 'presence'
RESPONSE = 'response'
ERROR = 'error'
DATA = 'data'
ALERT = 'alert'
MESSAGE = 'message'
MESSAGE_TEXT = 'message_text'
GET_ALL_CLIENTS = 'get_all_contacts'
GET_MY_CONTACTS = 'get_my_contacts'
ADD_CONTACT = 'add_contact'
EXIT = 'exit'

RESPONSE_200 = {RESPONSE: 200}
RESPONSE_202 = {
    RESPONSE: 202,
    ALERT: []
}
RESPONSE_400 = {
    RESPONSE: 400,
    ERROR: None
}

RESPONSE_511 = {
    RESPONSE: 511,
    DATA: None
}


from PyQt5.QtWidgets import QDialog, QPushButton, QLineEdit, QApplication, QLabel, qApp
from PyQt5.QtCore import QEvent

from client_gui.sign_up_window import SignUpDialog


class UserNameDialog(QDialog):
    """Окно авторизации для входа в приложение."""
    def __init__(self):
        super().__init__()
        self.auth = False
        self.reg = False
        self.name_pass = None
        self.name_pass_reg = None

        self.setWindowTitle('Привет')
        self.setFixedSize(230, 170)
        self.input_name_label = QLabel('Введите имя пользователя', self)
        self.input_name_label.move(40, 10)
        self.input_name_label.setFixedSize(220, 20)

        self.input_name = QLineEdit(self)
        self.input_name.setFixedSize(210, 27)
        self.input_name.move(10, 30)

        self.input_password_label = QLabel('Введите пароль', self)
        self.input_password_label.move(70, 60)
        self.input_password_label.setFixedSize(220, 20)

        self.input_password = QLineEdit(self)
        self.input_password.setFixedSize(210, 27)
        self.input_password.move(10, 80)

        self.ok_btn = QPushButton('Sign in', self)
        self.ok_btn.setFixedSize(60, 30)
        self.ok_btn.move(10, 120)
        self.ok_btn.clicked.connect(self.click)

        self.register_btn = QPushButton('Sign up', self)
        self.register_btn.setFixedSize(60, 30)
        self.register_btn.move(80, 120)
        self.register_btn.clicked.connect(self.reg_user)

        self.cancel_btn = QPushButton('Cancel', self)
        self.cancel_btn.setFixedSize(70, 30)
        self.cancel_btn.move(151, 120)
        self.cancel_btn.clicked.connect(qApp.exit)


    def click(self):
        """Пытается авторизоваться с введенными данными."""
        if self.input_name.text() and self.input_password.text():
            self.auth = True
            qApp.exit()
            self.name_pass = self.input_name.text(), self.input_password.text()
            self.close()

    def reg_user(self):
        """Переходит к окну регистрации."""
        self.reg_window = SignUpDialog()
        self.reg_window.show()
        self.reg_window.exec_()
        self.name_pass_reg = self.reg_window.input_name.text(), self.reg_window.input_password.text()
        self.reg = self.reg_window.ok_pressed
        self.close()

if __name__ == '__main__':
    app = QApplication([])
    dial = UserNameDialog()
    dial.show()
    app.exec_()






"""Unit-тесты клиента"""

import sys
import os
import unittest

sys.path.append(os.path.join(os.getcwd(), '..'))
from common.variables import RESPONSE, ERROR, USER, ACCOUNT_NAME, TIME, ACTION, PRESENCE
from client import create_presence, process_response_ans


class TestClass(unittest.TestCase):

    def test_create_presence(self):
        test = create_presence()
        test[TIME] = 1.1
        self.assertEqual(test, {ACTION: PRESENCE, TIME: 1.1, USER: {ACCOUNT_NAME: 'Guest'}})

    def test_create_answer_200(self):
        self.assertEqual(process_response_ans({RESPONSE: 200}), '200 : OK')

    def test_create_answer_400(self):
        self.assertEqual(process_response_ans({RESPONSE: 400, ERROR: 'Bad Request'}), '400 : Bad Request')

    def test_create_answer_value_error(self):
        self.assertRaises(ValueError, process_response_ans, {ERROR: 'Bad Request'})


if __name__ == '__main__':
    unittest.main()



"""Возможные ошибки."""

class IncorrectDataRecivedError(Exception):
    """Принято некоректное сообщение."""
    def __str__(self):
        return 'Принято некорректное сообщение от удалённого компьютера.'


class ServerError(Exception):
    """Ошибка сервера."""
    def __init__(self, text):
        self.text = text

    def __str__(self):
        return self.text


class NonDictInputError(Exception):
    """Аргумент функции не словарь."""
    def __str__(self):
        return 'Аргумент функции должен быть словарём.'


class ReqFieldMissingError(Exception):
    """Отсутствие обязательного поля в словаре."""
    def __init__(self, missing_field):
        self.missing_field = missing_field

    def __str__(self):
        return f'В принятом словаре отсутствует обязательное поле {self.missing_field}.'



import logging
import os

log = logging.getLogger('client')

PATH = os.path.dirname(os.path.abspath(__file__))
PATH = os.path.join(PATH, 'app.main.log')
formatter = logging.Formatter("%(asctime)s - %(levelname)-8s - %(module)s - %(message)s")

fh = logging.FileHandler(PATH, encoding='utf-8')
fh.setLevel(logging.DEBUG)
fh.setFormatter(formatter)

log.addHandler(fh)
log.setLevel(logging.DEBUG)

if __name__ == '__main__':
    cons = logging.StreamHandler()
    cons.setLevel(logging.DEBUG)
    cons.setFormatter(formatter)
    log.addHandler(cons)
    log.info('Тестовый запуск')



"""Графический интерфейс для работы с клиентом."""
import time
from datetime import datetime

from PyQt5.QtWidgets import QMainWindow, qApp, QMessageBox, QApplication, QListView, QHBoxLayout, QWidget, QListWidget, \
    QListWidgetItem, QPushButton, QVBoxLayout, QTextEdit, QLabel, QTableWidget, QTableWidgetItem
from PyQt5.QtCore import Qt, pyqtSlot, QEvent
from PyQt5.QtGui import QStandardItemModel, QStandardItem, QBrush, QColor
from client_gui.add_contact_gui import AddContactDialog
from errors import ServerError


class ClientMainWindow(QMainWindow):
    """Основное окно."""
    def __init__(self, database, transport):
        super().__init__()
        self.db = database
        self.transport = transport
        self.initUI()

    def initUI(self):
        self.setWindowTitle('Чат')
        self.setFixedSize(600, 600)
        self.centralwidget = QWidget(self)
        self.setCentralWidget(self.centralwidget)
        self.horizontal_layout = QHBoxLayout(self.centralwidget)

        # Список контактов и кнопки добавления, обновления и удаления.
        self.left_layout = QVBoxLayout()
        self.contacts_list_label = QLabel('Список контактов', self)
        self.contacts_list = QListWidget()

        self.refresh_btn = QPushButton('Обновить', self)
        self.del_btn = QPushButton('Удалить', self)
        self.add_btn = QPushButton('Добавить', self)
        self.bth_left_layout = QHBoxLayout()
        self.bth_left_layout.addWidget(self.refresh_btn)
        self.bth_left_layout.addWidget(self.del_btn)
        self.bth_left_layout.addWidget(self.add_btn)

        self.left_layout.addWidget(self.contacts_list_label)
        self.left_layout.addWidget(self.contacts_list)
        self.left_layout.addLayout(self.bth_left_layout)

        # Чат, окно набора сообщения и кнопка отправить.

        self.right_layout = QVBoxLayout()
        self.messages_list_label = QLabel('Окно чата', self)
        self.messages_list = QListView()
        self.messages_list.setWordWrap(True)


        self.text_msg_label = QLabel('Введите сообщение', self)
        self.text_msg = QTextEdit()
        self.text_msg.setDisabled(True)
        self.bth_right_layout = QHBoxLayout()
        self.clear_btn = QPushButton('Очистить окно', self)
        self.clear_btn.setDisabled(True)
        self.send_btn = QPushButton('Отправить', self)
        self.send_btn.setDisabled(True)
        self.bth_right_layout.addWidget(self.clear_btn)
        self.bth_right_layout.addWidget(self.send_btn)

        self.right_layout.addWidget(self.messages_list_label)
        self.right_layout.addWidget(self.messages_list)
        self.right_layout.addWidget(self.text_msg_label)
        self.right_layout.addWidget(self.text_msg)
        self.right_layout.addLayout(self.bth_right_layout)

        self.horizontal_layout.addLayout(self.left_layout)
        self.horizontal_layout.addLayout(self.right_layout)

        # Дополнительные требующиеся атрибуты
        self.contacts_model = None
        self.history_model = None
        self.messages = QMessageBox()
        self.current_chat = None

        # Действие кнопки "Обновить".
        self.refresh_btn.clicked.connect(lambda: self.refresh_contacts(self.transport.name))
        # Действие кнопки "Удалить".
        self.del_btn.clicked.connect(self.del_contacts)
        # Действие кнопки "Добавить".
        self.add_btn.clicked.connect(lambda: self.add_contact_window(self.db, self.transport.name))

        # Двойной клик по имени пользователя
        self.contacts_list.doubleClicked.connect(self.select_active_user)

        # Действие кнопки "Очистить окно".
        self.clear_btn.clicked.connect(self.clear_message_window)

        # Действие кнопки "Отправить".
        self.send_btn.clicked.connect(self.send_message)


        self.refresh_contacts(self.transport.name)
        self.show()

    def refresh_contacts(self, username):
        """Обновляет контакты пользователя."""
        self.contacts_list.clear()
        contact_list = self.db.get_my_contacts(username)
        self.contacts_list.addItems(contact_list)

    def add_contact_window(self, db, username):
        """Вызов окна для добавления контакта в список контактов."""
        add_dialog = AddContactDialog(db, username)
        add_dialog.add_btn.clicked.connect(lambda: self.add_contact_action(add_dialog))
        add_dialog.show()

    def add_contact_action(self, item):
        """Добавление контакта в список контактов."""
        new_contact = item.selector.currentText()
        self.add_contact(self.transport.name, new_contact)
        item.close()

    def add_contact(self, username, new_contact):
        """Добавление контакта в список контактов базы данных."""
        self.db.insert_in_contactdb(username, new_contact, '', str(datetime.now()))
        self.refresh_contacts(username)

    def del_contacts(self):
        """Удаление контакта из списка контактов и из базы данных."""
        try:
            name_for_del = self.contacts_list.currentItem().text()
            if name_for_del:
                self.db.del_in_contactdb(self.transport.name, name_for_del)
                self.refresh_contacts(self.transport.name)
                return
        except Exception:
            self.messages.warning(self, 'Ошибка', 'Вы не выбрали клиента')




    def select_active_user(self):
        """Данные выбранного пользователя. Вызов функции активации."""
        # Выбранный пользователем (даблклик) находится в выделеном элементе в QListView
        self.current_chat = self.contacts_list.currentIndex().data()

        # вызываем основную функцию
        self.set_active_user()

    def set_active_user(self):
        """Функция активации. Открывает доступ к управлению окном сообщения и кнопками управления им."""
        # активируем кнопки
        self.clear_btn.setDisabled(False)
        self.send_btn.setDisabled(False)
        self.text_msg.setDisabled(False)

        # Заполняем окно историю сообщений по требуемому пользователю.
        self.history_list_update()

    def history_list_update(self):
        """Заполняет окно истории сообщений"""
        msg_list = sorted(self.db.get_all_users_messages(self.current_chat), key=lambda item: item[3])

        if not self.history_model:
            self.history_model = QStandardItemModel()
            self.messages_list.setModel(self.history_model)
        # Очистим от старых записей
        self.history_model.clear()

        # Берём не более 20 последних записей.
        messages_list_length = len(msg_list)
        start_index = 0
        if messages_list_length > 20:
            start_index = messages_list_length - 20
        # Заполнение модели записями, так-же стоит разделить входящие и исходящие выравниванием и разным фоном.
        # Записи в обратном порядке, поэтому выбираем их с конца и не более 20
        for i in range(start_index, messages_list_length):
            item = msg_list[i]
            if item[2]:
                if item[0] == self.current_chat and item[1] == self.transport.name:
                    mess = QStandardItem(f'Входящее от {item[0]}:\n {item[2]}')
                    mess.setEditable(False)
                    mess.setBackground(QBrush(QColor(255, 213, 213)))
                    mess.setTextAlignment(Qt.AlignLeft)
                    self.history_model.appendRow(mess)
                elif item[1] == self.current_chat and item[0] == self.transport.name:
                    mess = QStandardItem(f'Исходящее от {item[0]}:\n {item[2]}')
                    mess.setEditable(False)
                    mess.setTextAlignment(Qt.AlignRight)
                    mess.setBackground(QBrush(QColor(204, 255, 204)))
                    self.history_model.appendRow(mess)

        self.messages_list.scrollToBottom()

    def clear_message_window(self):
        """Очищает окно сообщения."""
        self.text_msg.clear()

    def send_message(self):
        """Отправляет сообщение."""
        # Текст в поле, проверяем что поле не пустое затем забирается сообщение и поле очищается
        message_text = self.text_msg.toPlainText()
        self.text_msg.clear()
        if not message_text:
            return
        try:
            self.transport.send_message(self.transport.name, self.current_chat, message_text)
        except ServerError as err:
            self.messages.critical(self, 'Ошибка', err.text)
        except OSError as err:
            if err.errno:
                self.messages.critical(self, 'Ошибка', 'Потеряно соединение с сервером!')
                self.close()
            self.messages.critical(self, 'Ошибка', 'Таймаут соединения!')
        except (ConnectionResetError, ConnectionAbortedError):
            self.messages.critical(self, 'Ошибка', 'Потеряно соединение с сервером!')
            self.close()
        else:
            self.history_list_update()


    @pyqtSlot(str)
    def message(self, sender):
        """Открывает окно информации при получении сообщения от другого пользователя."""
        if sender == self.current_chat:
            self.history_list_update()
        else:
            # Проверим есть ли такой пользователь у нас в контактах:
            if sender in self.db.get_my_contacts(self.transport.name):
                # Если есть, спрашиваем и желании открыть с ним чат и открываем при желании
                if self.messages.question(self, 'Новое сообщение',
                                          f'Получено новое сообщение от {sender}, открыть чат с ним?', QMessageBox.Yes,
                                          QMessageBox.No) == QMessageBox.Yes:
                    self.current_chat = sender
                    self.set_active_user()
            else:
                print('NO')
                # Раз нету,спрашиваем хотим ли добавить юзера в контакты.
                if self.messages.question(self, 'Новое сообщение',
                                          f'Получено новое сообщение от {sender}.\n Данного пользователя нет в вашем контакт-листе.\n Добавить в контакты и открыть чат с ним?',
                                          QMessageBox.Yes,
                                          QMessageBox.No) == QMessageBox.Yes:
                    self.add_contact(self.transport.name, sender)
                    self.current_chat = sender
                    self.set_active_user()

    @pyqtSlot()
    def connection_lost(self):
        """Открывает окно предупреждения при потере соединения с сервером."""
        self.messages.warning(self, 'Сбой соединения', 'Потеряно соединение с сервером. ')
        self.close()

    def make_connection(self, trans_obj):
        """Создает связи с внешними сигналами получения новых сообщений и потере соединения."""
        trans_obj.new_message.connect(self.message)
        trans_obj.connection_lost.connect(self.connection_lost)


if __name__ == '__main__':
    app = QApplication([])
    app.setStyle('Fusion')
    main_window = ClientMainWindow()
    main_window.show()
    app.exec_()



from PyQt5.QtWidgets import QDialog, QPushButton, QLineEdit, QApplication, QLabel, qApp
from PyQt5.QtCore import QEvent


class SignUpDialog(QDialog):
    """Окно регистрации пользователя."""
    def __init__(self):
        super().__init__()
        self.ok_pressed = False

        self.setWindowTitle('Окно регистрации')
        self.setFixedSize(230, 210)
        self.input_name_label = QLabel('Введите имя пользователя', self)
        self.input_name_label.move(40, 10)
        self.input_name_label.setFixedSize(220, 20)

        self.input_name = QLineEdit(self)
        self.input_name.setFixedSize(210, 27)
        self.input_name.move(10, 30)

        self.input_password_label = QLabel('Введите пароль', self)
        self.input_password_label.move(70, 60)
        self.input_password_label.setFixedSize(220, 20)

        self.input_password = QLineEdit(self)
        self.input_password.setFixedSize(210, 27)
        self.input_password.move(10, 80)

        self.input_repeat_password_label = QLabel('Повторите пароль', self)
        self.input_repeat_password_label.move(70, 110)
        self.input_repeat_password_label.setFixedSize(220, 20)

        self.input_repeat_password = QLineEdit(self)
        self.input_repeat_password.setFixedSize(210, 27)
        self.input_repeat_password.move(10, 130)

        self.ok_btn = QPushButton('Ok', self)
        self.ok_btn.setFixedSize(60, 30)
        self.ok_btn.move(10, 170)
        self.ok_btn.clicked.connect(self.click)

        self.cancel_btn = QPushButton('Cancel', self)
        self.cancel_btn.setFixedSize(70, 30)
        self.cancel_btn.move(151, 170)
        self.cancel_btn.clicked.connect(qApp.exit)



    def click(self):
        """Подтверждает корректность введенных данных и закрывает окно."""
        if self.input_name.text() and self.input_password.text() \
                and self.input_repeat_password_label.text() \
                and self.input_password.text() == self.input_repeat_password.text():
            self.ok_pressed = True
            qApp.exit()


if __name__ == '__main__':
    app = QApplication([])
    dial = SignUpDialog()
    app.exec_()


"""Unit-тесты сервера"""

import sys
import os
import unittest
sys.path.append(os.path.join(os.getcwd(), '..'))
from common.variables import RESPONSE, ERROR, USER, ACCOUNT_NAME, TIME, ACTION, PRESENCE
from server import process_client_message

class TestServer(unittest.TestCase):
    msg_ok = {RESPONSE: 200}
    msg_error = {RESPONSE: 400,
                 ERROR: 'Bad Request'}

    def test_ok(self):
        self.assertEqual(process_client_message(
            {ACTION: PRESENCE, TIME: 1, USER: {ACCOUNT_NAME: 'Guest', STATUS: "Привет, я тут!"}}), self.msg_ok)

    def test_wrong_action(self):
        self.assertEqual(process_client_message(
            {ACTION: 'Error', TIME: 1, USER: {ACCOUNT_NAME: 'Guest', STATUS: "Привет, я тут!"}}), self.msg_error)

    def test_no_action(self):
        self.assertEqual(process_client_message(
            {TIME: 1, USER: {ACCOUNT_NAME: 'Guest', STATUS: "Привет, я тут!"}}), self.msg_error)

    def test_no_time_error(self):
        self.assertEqual(process_client_message(
            {ACTION: PRESENCE, USER: {ACCOUNT_NAME: 'Guest', STATUS: "Привет, я тут!"}}), self.msg_error)

    def test_wrong_account_name(self):
        self.assertEqual(process_client_message(
            {ACTION: PRESENCE, TIME: 1, USER: {ACCOUNT_NAME: 'Not guest', STATUS: "Привет, я тут!"}}), self.msg_error)

    def test_no_user(self):
        self.assertEqual(process_client_message(
            {ACTION: PRESENCE, TIME: 1}), self.msg_error)

if __name__ == '__main__':
    unittest.main()



from setuptools import setup, find_packages

setup(name="mess_client_redkin",
      version="0.0.1",
      description="Messenger Client",
      author="Ivan Redkin",
      author_email="stazhor@bk.ru",
      packages=find_packages(),
      install_requires=['PyQt5', 'sqlalchemy', 'pycryptodome', 'pycryptodomex'],
      )



"""Программа-клиент. Разработал Редкин И.А. декабрь 2022 г."""
import dis
import os
import sys
import json
import time
from socket import socket, AF_INET, SOCK_STREAM
from datetime import datetime
import logging
import threading
import hashlib
import hmac
import binascii

from Crypto.PublicKey import RSA
from PyQt5.QtCore import QObject, pyqtSignal
from PyQt5.QtWidgets import QApplication

import logs.config_client_log
from client_gui.client_start_gui import UserNameDialog
from client_gui.main_window_gui import ClientMainWindow
from common.variables import DEFAULT_PORT, DEFAULT_IP_ADDRESS, ACTION, \
    TIME, USER, ACCOUNT_NAME, FROM, PRESENCE, RESPONSE, \
    ERROR, MESSAGE, MESSAGE_TEXT, TO, EXIT, GET_ALL_CLIENTS, ALERT, GET_MY_CONTACTS, PUBLIC_KEY, AUTH, REG, STATE, \
    PASSWORD_HASH, DATA, RESPONSE_511
from common.utils import get_msg, send_msg, log
from errors import IncorrectDataRecivedError, ReqFieldMissingError, ServerError

from server_db import ServerStorage

CLIENT_LOGGER = logging.getLogger('client')
socket_lock = threading.Lock()


@log
def arg_parser():
    """Разбор аргументов командной строки при заходе в приложение.
    -a - адрес;
    -p - порт;
    -n - имя клиента;
    -pswd - пароль.
    """
    try:
        if '-p' in sys.argv:
            port = int(sys.argv[sys.argv.index('-p') + 1])
        else:
            port = DEFAULT_PORT
        if port < 1024 or port > 65535:
            raise ValueError
    except IndexError:
        CLIENT_LOGGER.error("При запуске клиента после параметра '-p' не указан номер порта.")
        sys.exit()
    except ValueError:
        CLIENT_LOGGER.error('При запуске клиента неверно указан номер порта.')
        sys.exit(1)

    try:
        if '-a' in sys.argv:
            address = sys.argv[sys.argv.index('-a') + 1]
        else:
            address = DEFAULT_IP_ADDRESS
    except IndexError:
        CLIENT_LOGGER.error("При запуске клиента после параметра '-a' не указан адрес.")
        sys.exit(1)

    try:
        if '-n' in sys.argv:
            name = sys.argv[sys.argv.index('-n') + 1]
        else:
            name = None
    except IndexError:
        CLIENT_LOGGER.error("При запуске клиента после параметра '-n' не указано имя.")
        sys.exit(1)

    try:
        if '-pswd' in sys.argv:
            password = sys.argv[sys.argv.index('-n') + 1]
        else:
            password = None
    except IndexError:
        CLIENT_LOGGER.error("При запуске клиента после параметра '-pswd' не указано имя.")
        sys.exit(1)

    return address, port, name, password


class ClientVerifier(type):
    """Метакласс. Проверка на корректность класса клиента.
    Вызов accept должен отсутствовать для сокетов клиента.
    Вызов listen должен отсутствовать для сокетов клиента.
    Необходимо использовать сокеты для работы по TCP протоколу.
    Сокеты на уровне классов создавать запрещено
    """

    def __new__(cls, future_class_name, future_class_parents, future_class_attrs):
        lst_argval = []
        for func in future_class_attrs:
            try:
                ret = dis.get_instructions(future_class_attrs[func])
            except TypeError:
                pass

            else:
                for i in ret:
                    lst_argval.append(i.argval)
        if 'accept' in lst_argval:
            raise 'Вызов accept должен отсутствовать для сокетов клиента'
        elif 'listen' in lst_argval:
            raise 'Вызов listen должен отсутствовать для сокетов клиента'
        if 'AF_INET' not in lst_argval or 'SOCK_STREAM' not in lst_argval:
            raise 'Необходимо использовать сокеты для работы по TCP протоколу'
        for val in future_class_attrs.values():
            if type(val) == socket:
                raise 'Сокеты на уровне классов создавать запрещено'

        return type.__new__(cls, future_class_name, future_class_parents, future_class_attrs)


class Client(threading.Thread, QObject):
    """Основной класс. Обменивается информацией с сервером."""
    my_contacts = []
    new_message = pyqtSignal(str)
    connection_lost = pyqtSignal()

    def __init__(self, port, ip_address, database, username, password, keys, state):
        threading.Thread.__init__(self)
        QObject.__init__(self)
        self.db = database
        self.port = port
        self.address = ip_address
        self.name = username
        self.password = password
        self.keys = keys
        self.state = state
        self.passwd_hash = ''
        self.running = True
        self.connection_init()

    def connection_init(self):
        """Установка соединение с сервером. Выявление ошибок при подключении и передачи приветственного сообщения."""
        print(f'Консольный месседжер. Клиентский модуль. Имя пользователя: {self.name}')
        CLIENT_LOGGER.info(
            f'Запущен клиент с параметрами: адрес сервера: {self.address}, '
            f'порт: {self.port}, имя пользователя: {self.name}')

        try:
            self.sock = socket(AF_INET, SOCK_STREAM)
            self.sock.connect((self.address, self.port))
            send_msg(self.sock, self.create_presence(self.name, self.password, self.state))
            self.answer = self.process_response_ans(get_msg(self.sock))
            CLIENT_LOGGER.info(f'Установлено соединение с сервером. Ответ сервера: {self.answer}')
            print(f'Установлено соединение с сервером.')
        except json.JSONDecodeError:
            CLIENT_LOGGER.error('Не удалось декодировать полученную Json строку.')
            sys.exit(1)
        except ServerError as error:
            CLIENT_LOGGER.error(f'При установке соединения сервер вернул ошибку: {error.text}')
            sys.exit(1)
        except ReqFieldMissingError as missing_error:
            CLIENT_LOGGER.error(f'В ответе сервера отсутствует необходимое поле {missing_error.missing_field}')
            sys.exit(1)
        except (ConnectionRefusedError, ConnectionError):
            CLIENT_LOGGER.critical(
                f'Не удалось подключиться к серверу {self.address}:{self.port}, '
                f'конечный компьютер отверг запрос на подключение.')
            sys.exit(1)

    @log
    def create_exit_message(self, account_name):
        """Создает сообщение EXIT."""
        return {
            ACTION: EXIT,
            TIME: str(datetime.now()),
            ACCOUNT_NAME: account_name
        }

    @log
    def message_from_server(self, sock, my_username):
        """Принимает сообщение с сервера или список всех клиентов сервера."""
        while True:
            try:
                self.message = get_msg(sock)
                if ACTION in self.message and self.message[ACTION] == MESSAGE and \
                        FROM in self.message and TO in self.message \
                        and MESSAGE_TEXT in self.message and self.message[TO] == my_username:
                    print(f'\nПолучено сообщение от пользователя {self.message[FROM]}:'
                          f'\n{self.message[MESSAGE_TEXT]}')
                    CLIENT_LOGGER.info(f'Получено сообщение от пользователя {self.message[FROM]}:'
                                       f'\n{self.message[MESSAGE_TEXT]}')
                    self.new_message.emit(self.message[FROM])

                elif self.message[RESPONSE] == 202:
                    print(f'Получен следующий список всех клиентов сервера: {self.message[ALERT]}')
                else:
                    CLIENT_LOGGER.error(f'Получено некорректное сообщение с сервера: {self.message}')
            except IncorrectDataRecivedError:
                CLIENT_LOGGER.error(f'Не удалось декодировать полученное сообщение.')
            except (OSError, ConnectionError, ConnectionAbortedError,
                    ConnectionResetError, json.JSONDecodeError):
                CLIENT_LOGGER.critical(f'Потеряно соединение с сервером.')
                self.connection_lost.emit()
                break

    @log
    def create_message(self, sock, account_name='Guest'):
        """Создает сообщение для другого пользователя."""
        self.to_user = input('Введите получателя сообщения: ')
        self.message = input('Введите сообщение для отправки: ')
        self.message_dict = {
            ACTION: MESSAGE,
            FROM: account_name,
            TO: self.to_user,
            TIME: str(datetime.now()),
            MESSAGE_TEXT: self.message
        }
        CLIENT_LOGGER.debug(f'Сформирован словарь сообщения: {self.message_dict}')
        try:
            send_msg(sock, self.message_dict)
            CLIENT_LOGGER.info(f'Отправлено сообщение для пользователя {self.to_user}')
        except Exception as e:
            print(e)
            CLIENT_LOGGER.critical('Потеряно соединение с сервером.')
            sys.exit(1)

    def send_message(self, from_user, to_user, message):
        """Создает словорб сообщения и отправляет его на сервер."""
        self.message_dict = {
            ACTION: MESSAGE,
            FROM: from_user,
            TO: to_user,
            TIME: str(datetime.now()),
            MESSAGE_TEXT: message
        }
        CLIENT_LOGGER.debug(f'Сформирован словарь сообщения: {self.message_dict}')
        try:
            send_msg(self.sock, self.message_dict)
            CLIENT_LOGGER.info(f'Отправлено сообщение для пользователя {to_user}')
        except Exception as e:
            print(e)
            CLIENT_LOGGER.critical('Потеряно соединение с сервером.')
            sys.exit(1)

    @log
    def add_contact(self, account_name):
        """Добавляет контакта в список контактов."""
        self.contact_name = input('Введите логин пользователя для добавления его в список контактов: ')
        self.my_contacts = self.db.get_my_contacts(account_name)
        if self.contact_name == account_name:
            print(f'Вы ввели свой ник')
            return
        elif self.contact_name in self.my_contacts:
            print(f'Пользователь {self.contact_name} уже в вашем списке')
            return

        self.my_contacts.append(self.contact_name)
        self.db.insert_in_contactdb(account_name, self.contact_name, '', str(datetime.now()))
        print(f'Пользователь {self.contact_name} добавлен в ваш список контактов')

    @log
    def del_contact(self, account_name):
        """Удаляет клиента из списка контактов."""
        self.contact_name = input('Введите логи пользователя для удаления его из списка контактов: ')
        self.my_contacts = self.db.get_my_contacts(account_name)
        if self.contact_name == account_name:
            print(f'Вы ввели свой логин')
            return
        elif self.contact_name not in self.my_contacts:
            print(f'Пользователя {self.contact_name} нет в вашем списке')
            return

        self.my_contacts.remove(self.contact_name)
        self.db.del_in_contactdb(account_name, self.contact_name)
        print(f'Пользователь {self.contact_name} удален из вашего списка контактов')

    @log
    def get_message_history(self, account_name):
        """Выдает список всех сообщений из базы данных."""
        return self.db.get_all_users_messages(account_name)

    @log
    def get_all_clients(self, sock, account_name):
        """Отправляет запрос на сервер для получения списка всех контактов."""
        self.get_contacts_dict = {
            ACTION: GET_ALL_CLIENTS,
            TIME: str(datetime.now()),
            ACCOUNT_NAME: account_name
        }
        CLIENT_LOGGER.debug(f'Сформирован словарь для получения списка всех контактов: {self.get_contacts_dict}')
        try:
            send_msg(sock, self.get_contacts_dict)
            CLIENT_LOGGER.info(f'Отправлен запрос на сервер для получения списка контактов')
        except Exception as e:
            print(e)
            CLIENT_LOGGER.critical('Потеряно соединение с сервером.')
            sys.exit(1)

    @log
    def user_interactive(self, sock, username):
        """Интерактивное управление запросами в терминале."""
        self.print_help()
        while True:
            self.command = input('Введите команду: ')
            if self.command == 'message':
                self.create_message(sock, username)
            elif self.command == 'help':
                self.print_help()
            elif self.command == 'get_all_clients':
                self.get_all_clients(sock, username)
                time.sleep(0.5)
            elif self.command == 'get_my_contacts':
                self.my_contacts = self.db.get_my_contacts(username)
                time.sleep(0.5)
            elif self.command == 'add_contact':
                self.add_contact(username)
                time.sleep(0.5)
            elif self.command == 'del_contact':
                self.del_contact(username)
                time.sleep(0.5)
            elif self.command == 'message_history':
                self.get_message_history(username)
                time.sleep(0.5)
            elif self.command == 'exit':
                send_msg(sock, self.create_exit_message(username))
                print('Завершение соединения.')
                CLIENT_LOGGER.info('Завершение работы по команде пользователя.')

                time.sleep(0.5)
                break
            else:
                print('Команда не распознана, попробойте снова. help - вывести поддерживаемые команды.')

    @log
    def create_presence(self, account_name, password, state):
        """Создает приветственное сообщение."""
        passwd_bytes = password.encode('utf-8')
        salt = account_name.lower().encode('utf-8')
        self.passwd_hash = hashlib.pbkdf2_hmac('sha512', passwd_bytes, salt, 10000)
        self.passwd_hash_str = binascii.hexlify(self.passwd_hash).decode('ascii')

        pubkey = self.keys.publickey().export_key().decode('ascii')
        out = {
            ACTION: PRESENCE,
            TIME: str(datetime.now()),
            USER: {
                ACCOUNT_NAME: account_name,
                PUBLIC_KEY: pubkey,
                PASSWORD_HASH: self.passwd_hash_str,
                STATE: state
            }
        }
        CLIENT_LOGGER.debug(f'Сформировано {PRESENCE} сообщение для пользователя {account_name}')
        return out

    def print_help(self):
        """Инструкция использования интерактивного управления через терминал."""
        print('Поддерживаемые команды:')
        print('message - отправить сообщение. Кому и текст будет запрошены отдельно.')
        print('help - вывести подсказки по командам')
        print('get_all_clients - получение списка всех подключенных клиентов')
        print('get_my_contacts - получение списка моих контактов')
        print('add_contact - добавление клиента в список моих контактов')
        print('del_contact - удаление клиента из списка моих контактов')
        print('message_history - история сообщений')
        print('exit - выход из программы')

    @log
    def process_response_ans(self, msg):
        """Разбирает приветственное сообщение от сервера."""
        CLIENT_LOGGER.debug(f'Разбор приветственного сообщения от сервера: {msg}')
        if RESPONSE in msg:
            if msg[RESPONSE] == 200:
                return '200 : OK'
            elif msg[RESPONSE] == 400:
                raise ServerError(f'400 : {msg[ERROR]}')
            elif msg[RESPONSE] == 511 :
                msg_data = msg[DATA]
                hash = hmac.new(self.passwd_hash_str.encode('ascii'), msg_data.encode('ascii'), 'MD5')
                digest = hash.digest()
                msg_to_server = RESPONSE_511
                msg_to_server[DATA] = binascii.b2a_base64(digest).decode('ascii')
                send_msg(self.sock, msg_to_server)
                return
        raise ReqFieldMissingError(RESPONSE)

    def run(self):
        '''Метод содержащий основной цикл работы транспортного потока.'''
        CLIENT_LOGGER.debug('Запущен процесс - приёмник собщений с сервера.')
        self.receiver = threading.Thread(target=self.message_from_server, args=(self.sock, self.name))
        self.receiver.daemon = True
        self.receiver.start()

        # self.user_interface = threading.Thread(target=self.user_interactive, args=(self.sock, self.name))
        # self.user_interface.daemon = True
        # self.user_interface.start()
        # CLIENT_LOGGER.debug('Запущены процессы')

        while True:
            time.sleep(1)
            if self.receiver.is_alive():
                continue
            break


if __name__ == '__main__':
    server_address, server_port, client_name, client_passwd = arg_parser()
    state = AUTH
    client_app = QApplication(sys.argv)
    client_app.setStyle('Fusion')

    if not client_name or not client_passwd:
        start_dialog = UserNameDialog()
        start_dialog.show()
        client_app.exec_()
        if start_dialog.name_pass:
            client_name = start_dialog.name_pass[0]
            client_passwd = start_dialog.name_pass[1]
            state = AUTH
            CLIENT_LOGGER.info(f'Клиент USERNAME = {client_name}, PASSWD = {client_passwd} пытается авторизоваться.')
        elif start_dialog.name_pass_reg:
            client_name = start_dialog.name_pass_reg[0]
            client_passwd = start_dialog.name_pass_reg[1]
            state = REG
            CLIENT_LOGGER.info(f'Регистрация USERNAME = {client_name}, PASSWD = {client_passwd}.')
        else:
            exit(0)

    # Загружаем ключи с файла, если же файла нет, то генерируем новую пару.
    dir_path = os.path.dirname(os.path.realpath(__file__))
    key_file = os.path.join(dir_path, f'{client_name}.key')
    if not os.path.exists(key_file):
        keys = RSA.generate(2048, os.urandom)
        with open(key_file, 'wb') as key:
            key.write(keys.export_key())
    else:
        with open(key_file, 'rb') as key:
            keys = RSA.import_key(key.read())


    database = ServerStorage()
    try:
        transport = Client(
            server_port,
            server_address,
            database,
            client_name,
            client_passwd,
            keys,
            state)
    except ServerError as e:
        print(e.text)
        exit(1)
    transport.setDaemon(True)
    transport.start()
    time.sleep(1)
    if state == AUTH:
        main_window = ClientMainWindow(database, transport)
        main_window.make_connection(transport)
        main_window.setWindowTitle(f'Чат {client_name}')
        client_app.exec_()


